@@ -1,18 +1,91 @@
-#ABSTRACT: Calculate N50 from a FASTA or FASTQ file without dependencies
+package Proch::N50;
+#ABSTRACT: a small module to calculate N50 (total size, and total number of sequences) for a FASTA or FASTQ file. It's easy to install, with minimal dependencies.
 
-use 5.016;
+use 5.014;
 use warnings;
 
-package Proch::N50;
-$Proch::N50::VERSION = '0.040';
-use JSON::PP;
+$Proch::N50::VERSION = '0.50';
 
+use JSON::PP;
+use FASTX::Reader;
 use File::Basename;
 use Exporter qw(import);
 our @EXPORT = qw(getStats getN50 jsonStats);
 
 
+sub getStats {
+    # Parses a FASTA/FASTQ file and returns stats
+    # Parameters:
+    # * filename (Str)
+    # * Also return JSON string (Bool)
+
+    my ( $file, $wantJSON ) = @_;
+    my $answer;
+    $answer->{status} = 1;
+    $answer->{N50}    = undef;
+
+    # Check file existence
+# uncoverable condition right
+    if ( !-e "$file" and $file ne '-' ) {
+        $answer->{status}  = 0;
+        $answer->{message} = "Unable to find <$file>";
+    }
+
+
+
+    # Return failed status if file not found or not readable
+    if ( $answer->{status} == 0 ) {
+        return $answer;
+    }
+
+    ##my @aux = undef;
+    my $Reader;
+    if ($file ne '-') {
+       $Reader = FASTX::Reader->new({ filename => "$file" });
+    } else {
+       $Reader = FASTX::Reader->new({ filename => '{{STDIN}}' });
+    }
+    my %sizes;
+    my ( $n, $slen ) = ( 0, 0 );
+
+    # Parse FASTA/FASTQ file
+    while ( my $seq = $Reader->getRead() ) {
+        ++$n;
+        my $size = length($seq->{seq});
+        $slen += $size;
+        $sizes{$size}++;
+    }
+
+    # Invokes core _n50fromHash() routine
+    my ($n50, $min, $max) = _n50fromHash( \%sizes, $slen );
+
+    my $basename = basename($file);
+
+    $answer->{N50}      = $n50 + 0;
+    $answer->{min}      = $min + 0;
+    $answer->{max}      = $max + 0;
+    $answer->{seqs}     = $n;
+    $answer->{size}     = $slen;
+    $answer->{filename} = $basename;
+    $answer->{dirname}  = dirname($file);
+
+    # If JSON is required return JSON
+    if ( defined $wantJSON ) {
+
+        my $json = JSON::PP->new->ascii->pretty->allow_nonref;
+        my $pretty_printed = $json->encode( $answer );
+        $answer->{json} = $pretty_printed;
+
+    }
+    return $answer;
+}
+
 sub _n50fromHash {
+    # _n50fromHash(): calculate stats from hash of lengths
+    #
+    # Parameters:
+    # * A hash of  key={contig_length} and value={no_contigs}
+    # * Sum of all contigs sizes
     my ( $hash_ref, $total ) = @_;
     my $tlen = 0;
     my @sorted_keys = sort { $a <=> $b } keys %{$hash_ref};
@@ -24,7 +97,9 @@ sub _n50fromHash {
     foreach my $s ( @sorted_keys ) {
         $tlen += $s * ${$hash_ref}{$s};
 
+     # N50 definition: https://en.wikipedia.org/wiki/N50_statistic
      # Was '>=' in my original implementation of N50. Now complies with 'seqkit'
+     # N50 Calculation
         return ($s, $min, $max) if ( $tlen > ( $total / 2 ) );
     }
 
@@ -35,127 +110,87 @@ sub getN50 {
     # Invokes the full getStats returning N50 or 0 in case of error;
     my ($file) = @_;
     my $stats = getStats($file);
+
+# Verify status and return
+# uncoverable branch false
     if ( $stats->{status} ) {
         return $stats->{N50};
-    }
-    else {
+    } else {
         return 0;
     }
 }
+
 sub jsonStats {
   my ($file) = @_;
   my $stats = getStats($file,  'JSON');
-  if ($stats->{status} and $stats->{json}) {
+
+# Return JSON object if getStats() was able to reduce one
+# uncoverable branch false
+  if (defined $stats->{json}) {
     return $stats->{json}
   } else {
+    # Return undef otherwise
     return undef;
   }
 }
-sub getStats {
 
-    # Parses a FASTA/FASTQ file and returns stats
-    my ( $file, $wantJSON ) = @_;
-    my $answer;
-    $answer->{status} = 1;
-    $answer->{N50}    = undef;
+# NOW READFQ IS PROVIDED BY: 'FASTX::Reader'
 
-    # Check file existence
-    if ( !-e "$file" and $file ne '-' ) {
-        $answer->{status}  = 0;
-        $answer->{message} = "Unable to find <$file>";
-    }
+# sub _readfq {
+#     # _readfq(): Heng Li's FASTA/FASTQ parser
+#     # Parameters:
+#     # * FileHandle
+#     # * Auxiliary array ref
+#     my ( $fh, $aux ) = @_;
+#     @$aux = [ undef, 0 ] if ( !(@$aux) );
+#
+#     # Parse FASTA/Q
+#     return if ( $aux->[1] );
+#     if ( !defined( $aux->[0] ) ) {
+#         while (<$fh>) {
+#             chomp;
+#             # Sequence header > or @
+#             if ( substr( $_, 0, 1 ) eq '>' || substr( $_, 0, 1 ) eq '@' ) {
+#                 $aux->[0] = $_;
+#                 last;
+#             }
+#         }
+#         if ( !defined( $aux->[0] ) ) {
+#             $aux->[1] = 1;
+#             return;
+#         }
+#     }
+#
+#     my $name = '';
+#     if ( defined $_ ) {
+#         $name = /^.(\S+)/ ? $1 : '';
+#     }
+#
+#     my $seq = '';
+#     my $c;
+#     $aux->[0] = undef;
+#     while (<$fh>) {
+#         chomp;
+#         $c = substr( $_, 0, 1 );
+#         last if ( $c eq '>' || $c eq '@' || $c eq '+' );
+#         $seq .= $_;
+#     }
+#     $aux->[0] = $_;
+#     $aux->[1] = 1 if ( !defined( $aux->[0] ) );
+#     return ( $name, $seq ) if ( $c ne '+' );
+#     my $qual = '';
+#     while (<$fh>) {
+#         chomp;
+#         $qual .= $_;
+#         if ( length($qual) >= length($seq) ) {
+#             $aux->[0] = undef;
+#             return ( $name, $seq, $qual );
+#         }
+#     }
+#     $aux->[1] = 1;
+#     return ( $name, $seq );
+# }
 
-    open FILE, '<', "$file" || do {
-        $answer->{status}  = 0;
-        $answer->{message} = "Unable to read <$file>";
-    };
-
-    if ( $answer->{status} == 0 ) {
-        return $answer;
-    }
-
-    my @aux = undef;
-    my %sizes;
-    my ( $n, $slen ) = ( 0, 0 );
-
-    # Parse FASTA/FASTQ file
-    while ( my ( $name, $seq ) = _readfq( \*FILE, \@aux ) ) {
-        ++$n;
-        my $size = length($seq);
-        $slen += $size;
-        $sizes{$size}++;
-    }
-    my ($n50, $min, $max) = _n50fromHash( \%sizes, $slen );
-
-    my $basename = basename($file);
-
-    $answer->{N50}      = $n50;
-    $answer->{min}      = $min;
-    $answer->{max}      = $max;
-    $answer->{seqs}     = $n;
-    $answer->{size}     = $slen;
-    $answer->{filename} = $basename;
-    $answer->{dirname}  = dirname($file);
-
-    if ( defined $wantJSON ) {
-
-        my $json = JSON::PP->new->ascii->pretty->allow_nonref;
-
-        my $pretty_printed = $json->encode( $answer );
-
-        $answer->{json} = $pretty_printed;
-
-    }
-    return $answer;
-}
-
-sub _readfq {
-    my ( $fh, $aux ) = @_;
-    @$aux = [ undef, 0 ] if ( !(@$aux) );
-    return if ( $aux->[1] );
-    if ( !defined( $aux->[0] ) ) {
-        while (<$fh>) {
-            chomp;
-            if ( substr( $_, 0, 1 ) eq '>' || substr( $_, 0, 1 ) eq '@' ) {
-                $aux->[0] = $_;
-                last;
-            }
-        }
-        if ( !defined( $aux->[0] ) ) {
-            $aux->[1] = 1;
-            return;
-        }
-    }
-
-    my $name = '';
-    if ( defined $_ ) {
-        $name = /^.(\S+)/ ? $1 : '';
-    }
-
-    my $seq = '';
-    my $c;
-    $aux->[0] = undef;
-    while (<$fh>) {
-        chomp;
-        $c = substr( $_, 0, 1 );
-        last if ( $c eq '>' || $c eq '@' || $c eq '+' );
-        $seq .= $_;
-    }
-    $aux->[0] = $_;
-    $aux->[1] = 1 if ( !defined( $aux->[0] ) );
-    return ( $name, $seq ) if ( $c ne '+' );
-    my $qual = '';
-    while (<$fh>) {
-        chomp;
-        $qual .= $_;
-        if ( length($qual) >= length($seq) ) {
-            $aux->[0] = undef;
-            return ( $name, $seq, $qual );
-        }
-    }
-    $aux->[1] = 1;
-    return ( $name, $seq );
-}
 
 1;
 
@@ -167,11 +202,11 @@ __END__
 
 =head1 NAME
 
-Proch::N50 - Calculate N50 from a FASTA or FASTQ file without dependencies
+Proch::N50 - a small module to calculate N50 (total size, and total number of sequences) for a FASTA or FASTQ file. It's easy to install, with minimal dependencies.
 
 =head1 VERSION
 
-version 0.040
+version 0.50
 
 =head1 SYNOPSIS
 
@@ -205,13 +240,11 @@ version 0.040
   #    "seqs" : 6,
   #    <...>
   #    "filename" : "small_test.fa",
-  #    "N50" : "65",
+  #    "N50" : 65,
   # }
-
-=head1 NAME
-
-B<Proch::N50> - a small module to calculate N50 (total size, and total number of
-sequences) for a FASTA or FASTQ file. It's small and without dependencies.
+  # Directly ask for the JSON object only:
+  my $json = jsonStats($filepath);
+  print $json; 
 
 =head1 METHODS
 
@@ -294,31 +327,53 @@ name of the directory containing the input file
 Returns the JSON string with basic stats (same as $result->{json} from I<getStats>(File, JSON)).
 Requires JSON::PP installed.
 
+=head2 _n50fromHash(hash, totalsize)
+
+This is an internal helper subroutine that perform the actual N50 calculation, hence its addition
+to the documentation.
+Expects the reference to an hash of sizes C<$size{SIZE} = COUNT> and the total sum of sizes obtained
+parsing the sequences file.
+Returns N50, min and max lengths.
+
 =head1 Dependencies
 
+=head2 Module (N50.pm)
+
 =over 4
 
-=item L<JSON::PP>
+=item L<FASTX::Reader> (required)
+
+=item L<JSON::PP>, <File::Basename> (core modules)
+
+=back
+
+=head2 Implementation (n50.pl)
+
+=over 4
+
+=item L<Term::ANSIColor> 
 
 =back
 
 =over 4
 
-=item L<Term::ANSIColor> (optional; for a demo script)
+=item L<JSON>
+
+(optional) when using C<--format JSON>
+
+=back
+
+=over 4
+
+=item L<Text::ASCIITable>
+
+(optional) when using C<--format screen>. This might be substituted by a different module in the future.
 
 =back
 
 =head1 AUTHOR
 
-Andrea Telatin <andrea@telatin.com>, Quadram Institute Bioscience
-
-=head1 COPYRIGHT AND LICENSE
-
-This free software under MIT licence. No warranty, explicit or implicit, is provided.
-
-=head1 AUTHOR
-
-Andrea Telatin <andrea.telatin@quadram.ac.uk>
+Andrea Telatin <andrea@telatin.com>
 
 =head1 COPYRIGHT AND LICENSE
 
